/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

 




package org.greenstone.gatherer.cdm;
/**************************************************************************************
 * Title:        Gatherer
 * Description:  The Gatherer: a tool for gathering and enriching a digital collection.
 * Copyright:    Copyright (c) 2001
 * Company:      The University of Waikato
 * Written:      03/05/02
 * Revised:      17/11/02 - Commented
 **************************************************************************************/
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Collections;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.CommandTokenizer;
import org.greenstone.gatherer.cdm.Index;
import org.greenstone.gatherer.msm.ElementWrapper;
import org.greenstone.gatherer.msm.MSMUtils;
import org.greenstone.gatherer.util.ExclusiveListSelectionListener;
import org.w3c.dom.Element;
/** This class is resposible for storing the indexes which have been assigned to this collection and the default index, and providing methods for interacting with both these data pools. It also knows how to turn itself into a String as it would be displayed in the collection configuration file.
* @author John Thompson, Greenstone Digital Library, University of Waikato
* @version 2.3
*/
public class IndexManager
	 extends DefaultListModel {
	 /** A reference to our creator, the collection design manager. */
	 private CollectionDesignManager manager = null;
	 /** The controls for editing the indexes. */
	 private Control controls = null;
	 /** A reference to ourselves so our inner methods have access. */
	 private DefaultListModel model = null;
	 /** A reference to the Gatherer, for access to the Dictionary and messaging purposes. */
	 private Gatherer gatherer = null;
	 /** The default index. */
	 private Index default_index = null;
	 /** Constructor.
	 * @param gatherer A reference to the <strong>Gatherer</strong>.
	 * @param manager A reference to the <strong>CollectionDesignManager</strong>.
	 */
	 public IndexManager(Gatherer gatherer, CollectionDesignManager manager) {
		  super();
		  this.gatherer = gatherer;
		  this.manager = manager;
		  this.model = this;
	 }
	 /** Method to add a new index.
	  * @param index The <strong>Index</strong> to add.
	  * @see org.greenstone.gatherer.Gatherer
	  * @see org.greenstone.gatherer.collection.CollectionManager
	  */
	 public void addIndex(Index index) {
		  if(!contains(index)) {
				String index_str = index.toString(false).toLowerCase();
				// Add alphabetically.
				for(int i = 0; i < size(); i++) {
					 Index sibling = (Index) get(i);
					 String sibling_str = sibling.toString(false).toLowerCase();
					 int position = index_str.compareTo(sibling_str);
					 // Sibling is before index.
					 if(position > 0) {
						  // Carry on.
					 }
					 // Index is equal to, or before sibling. Insert it.
					 else if(position == 0 || position < 0) {
						  add(i, index);
						  gatherer.c_man.configurationChanged();
						  return;
					 }
				}
				// If we got this far, we haven't inserted index, and we are out of model so.
				addElement(index);
				gatherer.c_man.configurationChanged();
		  }
		  else {
				JOptionPane.showMessageDialog(manager.gui, get("CDM.IndexManager.Index_Exists"), get("General.Warning"), JOptionPane.WARNING_MESSAGE);
		  }
	 }
	 /** Method to acquire the controls for editing the indexes.
	  * @return A <strong>JPanel</strong> containing the controls.
	  * @see org.greenstone.gatherer.cdm.IndexManager.Control
	  */
	 public JPanel getControls() {
		  if(controls == null) {
				controls = new Control();
		  }
		  return controls;
	 }
	 /** Method to get the default index.
	  * @return The default <strong>Index</strong>.
	  */
	 public Index getDefault() {
		  return default_index;
	 }
	 /** Method to retrieve a certain index, as referenced by an index number.
	  * @param index An <i>int</i> which indicates the position of the desired index.
	  * @return The <strong>Index</strong> at the given index, or <i>null</i> if no such index exists.
	  */
	 public Index getIndex(int index) {
		  if(0 <= index && index < size()) {
				return (Index)get(index);
		  }
		  return null;
	 }
	 /** Method to retrieve a certain index, given its name.
	  * @param name The name of the index as a <Strong>String</strong>.
	  * @return The <strong>Index</strong> that matches name, or <i>null</i> if no such index exists.
	  */
	 public Index getIndex(String name) {
		  ///ystem.err.println("Searching for index " + name);
		  for(int i = 0; i < size(); i++) {
				Index index = (Index) get(i);
				if(index.toString(false).equals(name)) {
					 ///ystem.err.println("Found.");
					 return (Index)get(i);
				}
		  }
		  ///ystem.err.println("No such index.");
		  return null;
	 }
	 /** A method to retrieve all of the indexes associated with this manager.
	  * @return A <strong>Vector</strong> of <strong>Index</strong>es.
	  */
	 public Vector getIndexes() {
		  Vector indexes = new Vector();
		  for(int i = 0; i < size(); i++) {
				indexes.add(get(i));
		  }
		  Collections.sort(indexes);
		  return indexes;
	 }
	/** Mark the current set of controls, if any, as obsolete and deallocate them. If further need of the controls will cause new controls to be created.
	* @see org.greenstone.gatherer.cdm.IndexManager.Control
	*/
	 public void invalidateControls() {
		  if(controls != null) {
				controls.destroy();
		  }
		  controls = null;
	 }
	 /** Method that attempts to parse an index related command from the given command. If such a command is parsed, it is immediately registered with this manager.
	  * @param command The <strong>String</strong> to parse.
	  * @return A <i>boolean</i> which is <i>true</i> if a command was parsed, <i>false</i> otherwise.
	  * @see org.greenstone.gatherer.cdm.CommandTokenizer
	  * @see org.greenstone.gatherer.cdm.Index
	  */
	 public boolean parse(String command) {
		  String temp = command.toLowerCase();
		  if(temp.startsWith("indexes")) {
				CommandTokenizer ct = new CommandTokenizer(command);
				ct.nextToken(); // Throw away indexes.
				while(ct.hasMoreTokens()) {
					 String entry = ct.nextToken();
					 if(entry.indexOf(":") != -1) {
						  String level_str = entry.substring(0, entry.indexOf(":"));
						  String sources_raw = entry.substring(entry.indexOf(":") + 1);
						  Vector sources = new Vector();
						  StringTokenizer st = new StringTokenizer(sources_raw, ",");
						  while(st.hasMoreTokens()) {
								String token = st.nextToken();
								// We may have to replace old : with whatever namespace separator we are using.
								token = token.replace(':', MSMUtils.NS_SEP);
								sources.add(token);
						  }
						  for(int i = 0; i < Index.LEVEL.length; i++) {
								if(level_str.equals(Index.LEVEL[i])) {
									 addIndex(new Index(i, sources, manager));
								}
						  }
					 }
					 else {
						  return false;
					 }
				}
				return true;
		  }
		  else if(temp.startsWith("defaultindex")) {
				CommandTokenizer ct = new CommandTokenizer(command);
				ct.nextToken();
				String entry = ct.nextToken();
				if(entry.indexOf(":") != -1) {
					 String level_str = entry.substring(0, entry.indexOf(":"));
					 String sources_raw = entry.substring(entry.indexOf(":") + 1);
					 Vector sources = new Vector();
					 StringTokenizer st = new StringTokenizer(sources_raw, ",");
					 while(st.hasMoreTokens()) {
						  sources.add(st.nextToken());
					 }
					 for(int i = 0; i < Index.LEVEL.length; i++) {
						  if(level_str.equals(Index.LEVEL[i])) {
								setDefault(new Index(i, sources, manager));
						  }
					 }
				}
				else {
					 return false;
				}
				return true;
		  }
		  return false;
	 }

	 public void removeAll() {
		  removeAllElements();
		  default_index = null;
		  gatherer.c_man.configurationChanged();
	 }

	 /** Method to remove a certain index.
	  * @param index The <Strong>Index</strong> to remove.
	  * @see org.greenstone.gatherer.Gatherer
	  * @see org.greenstone.gatherer.cdm.CollectionDesignManager
	  * @see org.greenstone.gatherer.cdm.CollectionMetaManager
	  * @see org.greenstone.gatherer.collection.CollectionManager
	  */
	 public void removeIndex(Index index) {
		  if(index != null) {
				String name = index.getName();
				if(name != null) {
					 Language default_language = manager.languages.getDefaultLanguage();
					 CollectionMeta metadata = new CollectionMeta(manager, index, default_language, name);
					 manager.collectionmetadatum.removeMetadata(metadata);
				}
				removeElement(index);
				if(default_index != null && default_index.equals(index)) {
					 default_index = null;
				}
				gatherer.c_man.configurationChanged();
		  }
	 }
	 /** Method to set the default index.
	  * @param index The new default <strong>Index</strong>.
	  * @see org.greenstone.gatherer.Gatherer
	  * @see org.greenstone.gatherer.collection.CollectionManager
	  */
	 public void setDefault(Index index) {
		  default_index = index;
		  if(index != null && !contains(index)) {
				addElement(index);
		  }
		  gatherer.c_man.configurationChanged();
	 }
	 /** Method to print out the contents of this class as a string.
	  * @return A <strong>String</strong> just as it would appear in the colleciton configuration file.
	  */
	 public String toString() {
		  String text = "";
		  // First the indexes.
		  if(size() > 0) {
				text = "indexes      ";
				for(int i = 0; i < size(); i++) {
					 Index index = (Index) get(i);
					 text = text + index.toString(false);
					 if(i < size() - 1) {
						  text = text + " ";
					 }
				}
				// Now the default index if there is one, or just the first index
				// if there isn't
				if(default_index != null) {
					 text = text + "\ndefaultindex " + default_index.toString(false) + "\n";
				}
				text = text + "\n";
		  }
		  return text;
	 }
	 /** Overloaded to call get with both a key and an empty argument array.
	  * @param key A <strong>String</strong> which is mapped to a initial String within the ResourceBundle.
	  * @return A <strong>String</strong> which has been referenced by the key String and that either contains no argument fields, or has had the argument fields automatiically populated with formatting Strings of with argument String provided in the get call.
	  */
	 private String get(String key) {
		  return get(key, null);
	 }
	 /** Used to retrieve a property value from the Locale specific ResourceBundle, based upon the key and arguments supplied. If the key cannot be found or if some other part of the call fails a default (English) error message is returned. <BR>
	  * Here the get recieves a second argument which is an array of Strings used to populate argument fields, denoted {<I>n</I>}, within the value String returned. Note that argument numbers greater than or equal to 32 are automatically mapped to the formatting String named Farg<I>n</I>.
	  * @param key A <strong>String</strong> which is mapped to a initial String within the ResourceBundle.
	  * @param args A <strong>String[]</strong> used to populate argument fields within the complete String.
	  * @return A <strong>String</strong> which has been referenced by the key String and that either contains no argument fields, or has had the argument fields automatiically populated with formatting Strings of with argument String provided in the get call.
	  * @see org.greenstone.gatherer.Gatherer
	  * @see org.greenstone.gatherer.Dictionary
	  */
	 private String get(String key, String args[]) {
		  if(key.indexOf('.') == -1) {
				key = "CDM.IndexManager." + key;
		  }
		  return gatherer.dictionary.get(key, args);
	 }
	 /** This class creates a set of controls for editing the indexes. */
	 private class Control
		  extends JPanel {
			/** The default size of a label on this control. */
		  private Dimension LABEL_SIZE = new Dimension(120,25);
		  /** The button used to add a new index. */
		  private JButton add = null;
		  /** The button used to clear the default index. */
		  private JButton clear_default = null;
		  /** The button used to remove an index. */
		  private JButton remove = null;
		  /** The button used to set the default index. */
		  private JButton set_default = null;
		  /** The combobox used to adjust what level the new index will built on. */
		  private JComboBox level = null;
		  /** The label denoting the default index to be built for this collection. */
		  private JLabel default_label = null;
		  /** The label denoting the list of assigned indexes. */
		  private JLabel index_label = null;
		  /** The label denoting the control for choosing index level. */
		  private JLabel level_label = null;
		  /** The label denoting the name of the index. */
		  private JLabel name_label = null;
		  /** The label denoting the list of possible sources for the index. */
		  private JLabel source_label = null;
		  /** The title shown at the top of the index controls. */
		  private JLabel title = null;
		  /** The list of assigned indexes. */
		  private JList index_list = null;
		  /** The list of possible sources on which the index could be built. */
		  private JList source_list = null;
		  /** The panel used to display the list of assigned indexes. */
		  private JPanel assigned_pane = null;
		  /** The panel which contains the buttons used to edit indexes. */
		  private JPanel button_pane = null;
		  /** The panel on which all other panels are displayed. */
		  private JPanel central_pane = null;
		  /** The control pane showing the editable controls. */
		  private JPanel control_pane = null;
		  /** The edit pane contains the list of possible sources. */
		  private JPanel edit_pane = null;
		  /** A panel used to correctly lay out the default index label. */
		  private JPanel default_pane = null;
		  /** The panel containing the title label and instructions. */
		  private JPanel header_pane = null;
		  /** The lvel panel contains the control for adjusting index level. */
		  private JPanel level_pane = null;
		  /** The pane in which the name control is set. */
		  private JPanel name_pane = null;
		  /** The scrollpane containing the instructions text area. */
		  private JScrollPane instructions_scroll = null;
		  /** A text area containing inline instructions. */
		  private JTextArea instructions = null;
		  /** A text field naming the default index. Non-editable. */
		  private JTextField default_value = null;
		  /** A text field for controlling the name of the new index. */
		  private JTextField name = null;
		  /** A model containing all of the available index sources. */
		  private Vector source_model = null;
		  /** Constructor.
			* @see org.greenstone.gatherer.Configuration
			* @see org.greenstone.gatherer.Gatherer
			* @see org.greenstone.gatherer.cdm.IndexManager.Control.AddListener
			* @see org.greenstone.gatherer.cdm.IndexManager.Control.ClearDefaultListener
			* @see org.greenstone.gatherer.cdm.IndexManager.Control.NameListener
			* @see org.greenstone.gatherer.cdm.IndexManager.Control.RemoveListener
			* @see org.greenstone.gatherer.cdm.IndexManager.Control.SetDefaultListener
			* @see org.greenstone.gatherer.collection.CollectionManager
			* @see org.greenstone.gatherer.gui.Coloring
			* @see org.greenstone.gatherer.msm.MetadataSetManager
			* @see org.greenstone.gatherer.util.ExclusiveListSelectionListener
			*/
		  public Control() {
				super();
				source_model = new Vector();
				source_model.add("text");
				source_model.addAll(gatherer.c_man.msm.getAssignedElements());
				//source_model.addAll(gatherer.c_man.msm.getElements());
				// Creation
				add = new JButton(get("Add"));
				add.setEnabled(false);
				add.setMnemonic(KeyEvent.VK_A);
				assigned_pane = new JPanel();
				button_pane = new JPanel();
				central_pane = new JPanel();
				control_pane = new JPanel();
				clear_default = new JButton(get("Clear_Default"));
				clear_default.setMnemonic(KeyEvent.VK_C);
				if(default_index == null) {
					 clear_default.setEnabled(false);
				}
				default_label = new JLabel(get("Default_Index"));
				default_pane = new JPanel();
				if(default_index != null) {
					 default_value = new JTextField(default_index.toString(true));
					 default_value.setCaretPosition(0);
				}
				else {
					 default_value = new JTextField();
				}
				edit_pane = new JPanel();
				header_pane = new JPanel();
				index_label = new JLabel(get("Indexes"));
				index_list = new JList(model);
				instructions = new JTextArea(get("Instructions"));
				instructions.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
				instructions.setEditable(false);
				instructions.setLineWrap(true);
				instructions.setRows(5);
				instructions.setWrapStyleWord(true);
				level = new JComboBox();
				level.addItem(get("Document"));
				level.addItem(get("Paragraph"));
				level.addItem(get("Section"));
				level.setEditable(false);
				level_label = new JLabel(get("Level"));
				level_label.setPreferredSize(LABEL_SIZE);
				level_pane = new JPanel();
				name = new JTextField();
				name_label = new JLabel(get("Name"));
				name_label.setPreferredSize(LABEL_SIZE);
				name_pane = new JPanel();
				remove = new JButton(get("Remove"));
				remove.setEnabled(false);
				remove.setMnemonic(KeyEvent.VK_R);
				set_default = new JButton(get("Set_Default"));
				set_default.setEnabled(false);
				set_default.setMnemonic(KeyEvent.VK_S);
				source_label = new JLabel(get("Source"));
				source_list = new JList(source_model);
				title = new JLabel(get("Title"));
				title.setHorizontalAlignment(JLabel.CENTER);

				// Listeners
				add.addActionListener(new AddListener());
				clear_default.addActionListener(new ClearDefaultListener());
				remove.addActionListener(new RemoveListener());
				set_default.addActionListener(new SetDefaultListener());
				name.addKeyListener(new NameListener());
				ListListener ll = new ListListener();
				index_list.addListSelectionListener(ll);
				source_list.addListSelectionListener(ll);
				ExclusiveListSelectionListener elsl = new ExclusiveListSelectionListener();
				elsl.add(index_list);
				elsl.add(source_list);

				// Layout
				title.setBorder(BorderFactory.createEmptyBorder(0,0,2,0));

				instructions.setBorder(BorderFactory.createEmptyBorder(2,5,2,5));

				header_pane.setLayout(new BorderLayout());
				header_pane.add(title, BorderLayout.NORTH);
				header_pane.add(new JScrollPane(instructions), BorderLayout.CENTER);

				name_label.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));

				name_pane.setBorder(BorderFactory.createEmptyBorder(4,2,4,2));
				name_pane.setLayout(new BorderLayout());
				name_pane.add(name_label, BorderLayout.WEST);
				name_pane.add(name, BorderLayout.CENTER);

				control_pane.setLayout(new BorderLayout());
				control_pane.add(source_label, BorderLayout.NORTH);
				control_pane.add(new JScrollPane(source_list), BorderLayout.CENTER);

				level_label.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));

				level_pane.setBorder(BorderFactory.createEmptyBorder(4,2,4,2));
				level_pane.setLayout(new BorderLayout());
				level_pane.add(level_label, BorderLayout.WEST);
				level_pane.add(level, BorderLayout.CENTER);

				edit_pane.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));
				edit_pane.setLayout(new BorderLayout());
				edit_pane.add(name_pane, BorderLayout.NORTH);
				edit_pane.add(control_pane, BorderLayout.CENTER);
				edit_pane.add(level_pane, BorderLayout.SOUTH);

				default_label.setBorder(BorderFactory.createEmptyBorder(0,0,0,5));

				default_pane.setBorder
					 (BorderFactory.createCompoundBorder
					  (BorderFactory.createCompoundBorder
						(BorderFactory.createEmptyBorder(2,2,2,2),
						 BorderFactory.createRaisedBevelBorder()),
						BorderFactory.createEmptyBorder(2,2,2,2)));
				default_pane.setLayout(new BorderLayout());
				default_pane.add(default_label, BorderLayout.WEST);
				default_pane.add(default_value, BorderLayout.CENTER);

				assigned_pane.setLayout(new BorderLayout());
				assigned_pane.add(index_label, BorderLayout.NORTH);
				assigned_pane.add(new JScrollPane(index_list), BorderLayout.CENTER);
				assigned_pane.add(default_pane, BorderLayout.SOUTH);

				central_pane.setBorder(BorderFactory.createEmptyBorder(2,0,2,0));
				central_pane.setLayout(new GridLayout(1,2));
				central_pane.add(edit_pane);
				central_pane.add(assigned_pane);

				button_pane.setLayout(new GridLayout(2,2));
				button_pane.add(add);
				button_pane.add(remove);
				button_pane.add(clear_default);
				button_pane.add(set_default);

				setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
				setLayout(new BorderLayout());
				add(header_pane, BorderLayout.NORTH);
				add(central_pane, BorderLayout.CENTER);
				add(button_pane, BorderLayout.SOUTH);
		  }
		  /* Destructor, removes persistant listeners from the Dictionary.
			*/
		  public void destroy() {
		  }
		  /** We override the updateUI method so that we can ensure we are scrolled to the top of the instructions box first.
		  * @see org.greenstone.gatherer.Gatherer
		  * @see org.greenstone.gatherer.collection.CollectionManager
		  * @see org.greenstone.gatherer.msm.MetadataSetManager
			*/
		  public void updateUI() {
				if(instructions != null) {
					 instructions.setCaretPosition(0);
				}
				// Reload the assigned indexes list.
				if(source_model != null) {
					 source_model.clear();
					 source_model.add("text");
					 source_model.addAll(gatherer.c_man.msm.getAssignedElements());
				}
				// Faster than a NPE its the - 'Super class'.
				super.updateUI();
		  }
		  /** Listens for actions apon the 'add' button in the IndexManager controls, and if detected calls the add method of the manager with a newly created index. */
		  private class AddListener
				implements ActionListener {
				/** Method called when an action is performed on a registered component, and when it does we check if we have enough data to create a new index, and if so we create one.
				* @param event An <strong>ActionEvent</strong> providing extra information about the event.
				* @see org.greenstone.gatherer.cdm.CollectionDesignManager
				* @see org.greenstone.gatherer.cdm.CollectionMeta
				* @see org.greenstone.gatherer.cdm.CollectionMetaManager
				* @see org.greenstone.gatherer.cdm.Index
				* @see org.greenstone.gatherer.cdm.Language
				* @see org.greenstone.gatherer.cdm.LanguageManager
				*/
				public void actionPerformed(ActionEvent event) {
					 if(!source_list.isSelectionEmpty() && name.getText().length() != 0) {
						  Object object[] = source_list.getSelectedValues();
						  Vector sources = new Vector();
						  for(int i = 0; i < object.length; i++) {
								sources.add(object[i]);
						  }
						  Index index = new Index(level.getSelectedIndex(), sources, manager);
						  // Before we add the index to the model, we have to add the collection metadata for this.
						  Language language = manager.languages.getDefaultLanguage();
						  CollectionMeta metadata = new CollectionMeta(manager, index, language, name.getText());
						  manager.collectionmetadatum.addMetadata(metadata);
						  // Finally add index.
						  addIndex(index);
					 }
				}
		  }
		  /** Listens for actions apon the 'clear default' button in the IndexManager controls, and if detected calls the setDefault method of the manager with <i>null</i>. */
		  private class ClearDefaultListener
				implements ActionListener {
				/** If called when an action occurs on a registered component, we clear the default index.
				* @param event An <strong>ActionEvent</strong> containing extra information about the action that occured.
				*/
				public void actionPerformed(ActionEvent event) {
					 setDefault(null);
					 clear_default.setEnabled(false);
					 default_value.setText("");
				}
		  }
		  /** Listens for actions apon the 'remove' button in the IndexManager controls, and if detected calls the remove method of the manager with the index selected for removal. */
		  private class RemoveListener
				implements ActionListener {
				/** If called when an action occurs on a registered component, we remove the currently selected index, if there is one.
				* @param event An <strong>ActionEvent</strong> containing extra information about the action that occured.
				* @see org.greenstone.gatherer.cdm.Index
				*/
				public void actionPerformed(ActionEvent event) {
					 if(!index_list.isSelectionEmpty()) {
						  removeIndex((Index)index_list.getSelectedValue());
						  if(default_index == null) {
								clear_default.setEnabled(false);
								default_value.setText("");
						  }
					 }
				}
		  }
		  /** Listens for actions apon the 'set default' button in the IndexManager controls, and if detected calls the setDefault method of the manager with the index selected for default. */
		  private class SetDefaultListener
				implements ActionListener {
				/** If called when an action occurs on a registered component, we set the default index to the index currently selected.
				* @param event An <strong>ActionEvent</strong> containing extra information about the action that occured.
				* @see org.greenstone.gatherer.cdm.Index
				*/
				public void actionPerformed(ActionEvent event) {
					 setDefault((Index)index_list.getSelectedValue());
					 if(default_index != null) {
						  clear_default.setEnabled(true);
						  default_value.setText(default_index.toString(true));
						  default_value.setCaretPosition(0);
					 }
					 else {
						  clear_default.setEnabled(false);
						  default_value.setText("");
					 }
				}
		  }
		  /** Listens for key presses within the name field, and enabled or disables controls as appropriate. */
		  private class NameListener
				extends KeyAdapter {
				/** Called when a key is released, this is the perfect time to enable the add button if the fields are appropriately set.
				* @param event A <strong>KeyEvent</strong> containing information about the key released.
				*/
				public void keyReleased(KeyEvent event) {
					 if(source_list.isSelectionEmpty() || name.getText().length() == 0) {
						  add.setEnabled(false);
					 }
					 else {
						  add.setEnabled(true);
					 }
					 if(index_list.isSelectionEmpty()) {
						  remove.setEnabled(false);
						  set_default.setEnabled(false);
					 }
					 else {
						  remove.setEnabled(true);
						  set_default.setEnabled(true);
					 }
				}
		  }
		  /** Listens for selections within the list on the IndexManager controls, and if a change is detected enables, or disables, controls appropriately. */
		  private class ListListener
				implements ListSelectionListener {
				/** This method is called whenever the source list selection changes. When it does we need to check if the add button should now be enabled.
				* @param event A <strong>ListSelectionEvent</strong> containing further information about the list selection.
				*/
				public void valueChanged(ListSelectionEvent event) {
					 if(source_list.isSelectionEmpty() || name.getText().length() == 0) {
						  add.setEnabled(false);
					 }
					 else {
						  add.setEnabled(true);
					 }
					 if(index_list.isSelectionEmpty()) {
						  remove.setEnabled(false);
						  set_default.setEnabled(false);
					 }
					 else {
						  remove.setEnabled(true);
						  set_default.setEnabled(true);
					 }
				}
		  }
	 }
}


